"""
Mocking pyEX data fetch api calls
"""

import pandas as pd


def chartDF(
        symbol,
        timeframe,
        date):
    """chartDF

    mock pyEX chartDF

    :param symbol: ticker symbol
    :param timeframe: timeframe argument
    :param date: date value
    """

    val = {
        'symbol': [symbol],
        'timeframe': [timeframe],
        'date': [date]
    }
    df = pd.DataFrame(
        val)
    return df
# end of chartDF


def stockStatsDF(
        symbol):
    """stockStatsDF

    mock pyEX stockStatsDF

    :param symbol: ticker symbol
    """

    val = {
        'symbol': [symbol],
        'testcase': ['mock-stockStatsDF']
    }
    df = pd.DataFrame(
        val)
    return df
# end of stockStatsDF


def peersDF(
        symbol):
    """peersDF

    mock pyEX peersDF

    :param symbol: ticker symbol
    """

    val = {
        'symbol': [symbol],
        'testcase': ['mock-peersDF']
    }
    df = pd.DataFrame(
        val)
    return df
# end of peersDF


def newsDF(
        symbol):
    """newsDF

    mock pyEX newsDF

    :param symbol: ticker symbol
    """

    val = {
        'symbol': [symbol],
        'testcase': ['mock-newsDF']
    }
    df = pd.DataFrame(
        val)
    return df
# end of newsDF


def financialsDF(
        symbol):
    """financialsDF

    mock pyEX financialsDF

    :param symbol: ticker symbol
    """

    val = {
        'symbol': [symbol],
        'testcase': ['mock-financialsDF']
    }
    df = pd.DataFrame(
        val)
    return df
# end of financialsDF


def earningsDF(
        symbol):
    """earningsDF

    mock pyEX earningsDF

    :param symbol: ticker symbol
    """

    val = {
        'symbol': [symbol],
        'testcase': ['mock-earningsDF']
    }
    df = pd.DataFrame(
        val)
    return df
# end of earningsDF


def dividendsDF(
        symbol):
    """dividendsDF

    mock pyEX dividendsDF

    :param symbol: ticker symbol
    """

    val = {
        'symbol': [symbol],
        'testcase': ['mock-dividendsDF']
    }
    df = pd.DataFrame(
        val)
    return df
# end of dividendsDF


def companyDF(
        symbol):
    """companyDF

    mock pyEX companyDF

    :param symbol: ticker symbol
    """

    val = {
        'symbol': [symbol],
        'testcase': ['mock-companyDF']
    }
    df = pd.DataFrame(
        val)
    return df
# end of companyDF



"""
Helpers and examples for supported API Requests that each Celery
Task supports:

- analysis_engine.work_tasks.get_new_pricing_data
- analysis_engine.work_tasks.handle_pricing_update_task
- analysis_engine.work_tasks.publish_pricing_update

"""

import datetime
import analysis_engine.iex.utils as iex_utils
from analysis_engine.consts import TICKER
from analysis_engine.consts import TICKER_ID
from analysis_engine.consts import DAILY_S3_BUCKET_NAME
from analysis_engine.consts import MINUTE_S3_BUCKET_NAME
from analysis_engine.consts import TICK_S3_BUCKET_NAME
from analysis_engine.consts import STATS_S3_BUCKET_NAME
from analysis_engine.consts import PEERS_S3_BUCKET_NAME
from analysis_engine.consts import NEWS_S3_BUCKET_NAME
from analysis_engine.consts import FINANCIALS_S3_BUCKET_NAME
from analysis_engine.consts import EARNINGS_S3_BUCKET_NAME
from analysis_engine.consts import DIVIDENDS_S3_BUCKET_NAME
from analysis_engine.consts import COMPANY_S3_BUCKET_NAME
from analysis_engine.consts import PREPARE_S3_BUCKET_NAME
from analysis_engine.consts import ANALYZE_S3_BUCKET_NAME
from analysis_engine.iex.consts import FETCH_DAILY
from analysis_engine.iex.consts import FETCH_MINUTE
from analysis_engine.iex.consts import FETCH_TICK
from analysis_engine.iex.consts import FETCH_STATS
from analysis_engine.iex.consts import FETCH_PEERS
from analysis_engine.iex.consts import FETCH_NEWS
from analysis_engine.iex.consts import FETCH_FINANCIALS
from analysis_engine.iex.consts import FETCH_EARNINGS
from analysis_engine.iex.consts import FETCH_DIVIDENDS
from analysis_engine.iex.consts import FETCH_COMPANY
from analysis_engine.iex.consts import DATAFEED_DAILY
from analysis_engine.iex.consts import DATAFEED_MINUTE
from analysis_engine.iex.consts import DATAFEED_TICK
from analysis_engine.iex.consts import DATAFEED_STATS
from analysis_engine.iex.consts import DATAFEED_PEERS
from analysis_engine.iex.consts import DATAFEED_NEWS
from analysis_engine.iex.consts import DATAFEED_FINANCIALS
from analysis_engine.iex.consts import DATAFEED_EARNINGS
from analysis_engine.iex.consts import DATAFEED_DIVIDENDS
from analysis_engine.iex.consts import DATAFEED_COMPANY


def build_get_new_pricing_request(
        label=None):
    """build_get_new_pricing_request

    Build a sample Celery task API request:
    analysis_engine.work_tasks.get_new_pricing_data

    Used for testing: run_get_new_pricing_data(work)

    :param label: log label to use
    """
    ticker = TICKER
    ticker_id = TICKER_ID
    base_key = '{}_{}'.format(
        ticker,
        datetime.datetime.utcnow().strftime(
            '%Y_%m_%d_%H_%M_%S'))
    s3_bucket_name = 'pricing'
    s3_key = base_key
    redis_key = base_key
    use_strike = None
    contract_type = 'C'
    get_pricing = True
    get_news = True
    get_options = True

    work = {
        'ticker': ticker,
        'ticker_id': ticker_id,
        's3_bucket': s3_bucket_name,
        's3_key': s3_key,
        'redis_key': redis_key,
        'strike': use_strike,
        'contract': contract_type,
        'get_pricing': get_pricing,
        'get_news': get_news,
        'get_options': get_options
    }

    if label:
        work['label'] = label

    return work
# end of build_get_new_pricing_request


def build_cache_ready_pricing_dataset(
        label=None):
    """build_cache_ready_pricing_dataset

    Build a cache-ready pricing dataset to replicate
    the ``get_new_pricing_data`` task

    :param label: log label to use
    """
    cache_data = {
        "news": [
            {
                "d": "16 hours ago",
                "s": "Yahoo Finance",
                "sp": "Some Title 1",
                "sru": "http://news.google.com/news/url?values",
                "t": "Some Title 1",
                "tt": "1493311950",
                "u": "http://finance.yahoo.com/news/url1",
                "usg": "ke1"
            },
            {
                "d": "18 hours ago",
                "s": "Yahoo Finance",
                "sp": "Some Title 2",
                "sru": "http://news.google.com/news/url?values",
                "t": "Some Title 2",
                "tt": "1493311950",
                "u": "http://finance.yahoo.com/news/url2",
                "usg": "key2"
            }
        ],
        "options": [
            {
                "ask": 0.0,
                "bid": 0.0,
                "change": 0.0,
                "contractSize": "REGULAR",
                "contractSymbol": "SPY170505C00015000",
                "currency": "USD",
                "expiration": 1493942400,
                "impliedVolatility": 0.2500075,
                "inTheMoney": False,
                "lastPrice": 0.28,
                "lastTradeDate": 1493323145,
                "openInterest": 0,
                "percentChange": 0.0,
                "strike": 286.0,
                "volume": 1106
            }
        ],
        "pricing": {
            "ask": 0.0,
            "askSize": 8,
            "averageDailyVolume10Day": 67116380,
            "averageDailyVolume3Month": 64572187,
            "bid": 0.0,
            "bidSize": 10,
            "close": 287.6,
            "currency": "USD",
            "esgPopulated": False,
            "exchange": "PCX",
            "exchangeDataDelayedBy": 0,
            "exchangeTimezoneName": "America/New_York",
            "exchangeTimezoneShortName": "EDT",
            "fiftyDayAverage": 285.21735,
            "fiftyDayAverageChange": 2.8726501,
            "fiftyDayAverageChangePercent": 0.010071794,
            "fiftyTwoWeekHigh": 291.74,
            "fiftyTwoWeekHighChange": -3.649994,
            "fiftyTwoWeekHighChangePercent": -0.012511119,
            "fiftyTwoWeekLow": 248.02,
            "fiftyTwoWeekLowChange": 40.069992,
            "fiftyTwoWeekLowChangePercent": 0.16155952,
            "fiftyTwoWeekRange": "248.02 - 291.74",
            "financialCurrency": "USD",
            "fullExchangeName": "NYSEArca",
            "gmtOffSetMilliseconds": -14400000,
            "high": 289.03,
            "language": "en-US",
            "longName": "SPDR S&amp;P 500 ETF",
            "low": 287.88,
            "market": "us_market",
            "marketCap": 272023797760,
            "marketState": "POSTPOST",
            "messageBoardId": "finmb_6160262",
            "open": 288.74,
            "postMarketChange": 0.19998169,
            "postMarketChangePercent": 0.06941398,
            "postMarketPrice": 288.3,
            "postMarketTime": 1536623987,
            "priceHint": 2,
            "quoteSourceName": "Delayed Quote",
            "quoteType": "ETF",
            "region": "US",
            "regularMarketChange": 0.48999023,
            "regularMarketChangePercent": 0.17037213,
            "regularMarketDayHigh": 289.03,
            "regularMarketDayLow": 287.88,
            "regularMarketDayRange": "287.88 - 289.03",
            "regularMarketOpen": 288.74,
            "regularMarketPreviousClose": 287.6,
            "regularMarketPrice": 288.09,
            "regularMarketTime": 1536609602,
            "regularMarketVolume": 50210903,
            "sharesOutstanding": 944232000,
            "shortName": "SPDR S&P 500",
            "sourceInterval": 15,
            "symbol": "SPY",
            "tradeable": True,
            "trailingThreeMonthNavReturns": 7.71,
            "trailingThreeMonthReturns": 7.63,
            "twoHundredDayAverage": 274.66153,
            "twoHundredDayAverageChange": 13.428467,
            "twoHundredDayAverageChangePercent": 0.048890963,
            "volume": 50210903,
            "ytdReturn": 9.84
        }
    }
    return cache_data
# end of build_cache_ready_pricing_dataset


def build_publish_pricing_request(
        label=None):
    """build_publish_pricing_request

    Build a sample Celery task API request:
    analysis_engine.work_tasks.publisher_pricing_update

    Used for testing: run_publish_pricing_update(work)

    :param label: log label to use
    """
    ticker = TICKER
    ticker_id = TICKER_ID
    base_key = '{}_{}'.format(
        ticker,
        datetime.datetime.utcnow().strftime(
            '%Y_%m_%d_%H_%M_%S'))
    s3_bucket_name = 'pricing'
    s3_key = base_key
    redis_key = base_key
    use_strike = None
    contract_type = 'C'
    use_data = build_cache_ready_pricing_dataset()

    work = {
        'ticker': ticker,
        'ticker_id': ticker_id,
        'strike': use_strike,
        'contract': contract_type,
        's3_bucket': s3_bucket_name,
        's3_key': s3_key,
        'redis_key': redis_key,
        'data': use_data
    }

    if label:
        work['label'] = label

    return work
# end of build_publish_pricing_request


def build_publish_from_s3_to_redis_request(
        label=None):
    """build_publish_from_s3_to_redis_request

    Build a sample Celery task API request:
    analysis_engine.work_tasks.publish_from_s3_to_redis

    Used for testing: run_publish_from_s3_to_redis(work)

    :param label: log label to use
    """
    ticker = TICKER
    ticker_id = TICKER_ID
    base_key = '{}_{}'.format(
        ticker,
        datetime.datetime.utcnow().strftime(
            '%Y_%m_%d_%H_%M_%S'))
    s3_bucket_name = 'pricing'
    s3_key = base_key
    redis_key = base_key
    s3_enabled = True
    redis_enabled = True

    work = {
        'ticker': ticker,
        'ticker_id': ticker_id,
        's3_bucket': s3_bucket_name,
        's3_key': s3_key,
        'redis_key': redis_key,
        's3_enabled': s3_enabled,
        'redis_enabled': redis_enabled
    }

    if label:
        work['label'] = label

    return work
# end of build_publish_from_s3_to_redis_request


def build_prepare_dataset_request(
        label=None):
    """build_prepare_dataset_request

    Build a sample Celery task API request:
    analysis_engine.work_tasks.prepare_pricing_dataset

    Used for testing: run_prepare_pricing_dataset(work)

    :param label: log label to use
    """
    ticker = TICKER
    ticker_id = TICKER_ID
    base_key = '{}_{}'.format(
        ticker,
        datetime.datetime.utcnow().strftime(
            '%Y_%m_%d_%H_%M_%S'))
    s3_bucket_name = 'pricing'
    s3_key = base_key
    redis_key = base_key
    s3_prepared_bucket_name = PREPARE_S3_BUCKET_NAME
    s3_prepared_key = '{}.csv'.format(
        base_key)
    redis_prepared_key = '{}'.format(
        base_key)
    ignore_columns = None
    s3_enabled = True
    redis_enabled = True

    work = {
        'ticker': ticker,
        'ticker_id': ticker_id,
        's3_bucket': s3_bucket_name,
        's3_key': s3_key,
        'redis_key': redis_key,
        'prepared_s3_key': s3_prepared_key,
        'prepared_s3_bucket': s3_prepared_bucket_name,
        'prepared_redis_key': redis_prepared_key,
        'ignore_columns': ignore_columns,
        's3_enabled': s3_enabled,
        'redis_enabled': redis_enabled
    }

    if label:
        work['label'] = label

    return work
# end of build_prepare_dataset_request


def build_analyze_dataset_request(
        label=None):
    """build_analyze_dataset_request

    Build a sample Celery task API request:
    analysis_engine.work_tasks.analyze_pricing_dataset

    Used for testing: run_analyze_pricing_dataset(work)

    :param label: log label to use
    """
    ticker = TICKER
    ticker_id = TICKER_ID
    base_key = '{}_{}'.format(
        ticker,
        datetime.datetime.utcnow().strftime(
            '%Y_%m_%d_%H_%M_%S'))
    s3_bucket_name = PREPARE_S3_BUCKET_NAME
    s3_key = base_key
    redis_key = base_key
    s3_analyzed_bucket_name = ANALYZE_S3_BUCKET_NAME
    s3_analyzed_key = '{}.csv'.format(
        base_key)
    redis_analyzed_key = '{}'.format(
        base_key)
    ignore_columns = None
    s3_enabled = True
    redis_enabled = True

    work = {
        'ticker': ticker,
        'ticker_id': ticker_id,
        's3_bucket': s3_bucket_name,
        's3_key': s3_key,
        'redis_key': redis_key,
        'analyzed_s3_key': s3_analyzed_key,
        'analyzed_s3_bucket': s3_analyzed_bucket_name,
        'analyzed_redis_key': redis_analyzed_key,
        'ignore_columns': ignore_columns,
        's3_enabled': s3_enabled,
        'redis_enabled': redis_enabled
    }

    if label:
        work['label'] = label

    return work
# end of build_analyze_dataset_request


"""
IEX API Requests
"""


def build_iex_fetch_daily_request(
        label=None):
    """build_iex_fetch_daily_request

    Fetch daily data from IEX

    :param label: log label to use
    """
    ticker = TICKER
    base_key = '{}_daily_{}'.format(
        ticker,
        datetime.datetime.utcnow().strftime(
            '%Y_%m_%d_%H_%M_%S'))
    s3_bucket_name = DAILY_S3_BUCKET_NAME
    s3_key = base_key
    redis_key = base_key
    s3_enabled = True
    redis_enabled = True

    work = {
        'ft_type': FETCH_DAILY,
        'fd_type': DATAFEED_DAILY,
        'ticker': ticker,
        'timeframe': '5y',
        's3_bucket': s3_bucket_name,
        's3_key': s3_key,
        'redis_key': redis_key,
        's3_enabled': s3_enabled,
        'redis_enabled': redis_enabled
    }

    if label:
        work['label'] = label

    return work
# end of build_iex_fetch_daily_request


def build_iex_fetch_minute_request(
        label=None):
    """build_iex_fetch_minute_request

    Fetch minute data from IEX

    :param label: log label to use
    """
    ticker = TICKER
    base_key = '{}_minute_{}'.format(
        ticker,
        datetime.datetime.utcnow().strftime(
            '%Y_%m_%d_%H_%M_%S'))
    s3_bucket_name = MINUTE_S3_BUCKET_NAME
    s3_key = base_key
    redis_key = base_key
    s3_enabled = True
    redis_enabled = True

    work = {
        'ft_type': FETCH_MINUTE,
        'fd_type': DATAFEED_MINUTE,
        'ticker': ticker,
        'timeframe': '1d',
        'from': iex_utils.last_month().strftime(
            '%Y-%m-%d %H:%M:%S'),
        'last_close': None,
        's3_bucket': s3_bucket_name,
        's3_key': s3_key,
        'redis_key': redis_key,
        's3_enabled': s3_enabled,
        'redis_enabled': redis_enabled
    }

    if label:
        work['label'] = label

    return work
# end of build_iex_fetch_minute_request


def build_iex_fetch_tick_request(
        label=None):
    """build_iex_fetch_tick_request

    Fetch tick data from IEX

    :param label: log label to use
    """
    ticker = TICKER
    base_key = '{}_tick_{}'.format(
        ticker,
        datetime.datetime.utcnow().strftime(
            '%Y_%m_%d_%H_%M_%S'))
    s3_bucket_name = TICK_S3_BUCKET_NAME
    s3_key = base_key
    redis_key = base_key
    s3_enabled = True
    redis_enabled = True

    work = {
        'ft_type': FETCH_TICK,
        'fd_type': DATAFEED_TICK,
        'ticker': ticker,
        'timeframe': '1d',
        'from': iex_utils.last_month().strftime(
            '%Y-%m-%d %H:%M:%S'),
        'last_close': None,
        's3_bucket': s3_bucket_name,
        's3_key': s3_key,
        'redis_key': redis_key,
        's3_enabled': s3_enabled,
        'redis_enabled': redis_enabled
    }

    if label:
        work['label'] = label

    return work
# end of build_iex_fetch_tick_request


def build_iex_fetch_stats_request(
        label=None):
    """build_iex_fetch_stats_request

    Fetch statistic data from IEX

    :param label: log label to use
    """
    ticker = TICKER
    base_key = '{}_stat_{}'.format(
        ticker,
        datetime.datetime.utcnow().strftime(
            '%Y_%m_%d_%H_%M_%S'))
    s3_bucket_name = STATS_S3_BUCKET_NAME
    s3_key = base_key
    redis_key = base_key
    s3_enabled = True
    redis_enabled = True

    work = {
        'ft_type': FETCH_STATS,
        'fd_type': DATAFEED_STATS,
        'ticker': ticker,
        'from': iex_utils.last_month().strftime(
            '%Y-%m-%d %H:%M:%S'),
        's3_bucket': s3_bucket_name,
        's3_key': s3_key,
        'redis_key': redis_key,
        's3_enabled': s3_enabled,
        'redis_enabled': redis_enabled
    }

    if label:
        work['label'] = label

    return work
# end of build_iex_fetch_stats_request


def build_iex_fetch_peers_request(
        label=None):
    """build_iex_fetch_peers_request

    Fetch peer data from IEX

    :param label: log label to use
    """
    ticker = TICKER
    base_key = '{}_peer_{}'.format(
        ticker,
        datetime.datetime.utcnow().strftime(
            '%Y_%m_%d_%H_%M_%S'))
    s3_bucket_name = PEERS_S3_BUCKET_NAME
    s3_key = base_key
    redis_key = base_key
    s3_enabled = True
    redis_enabled = True

    work = {
        'ft_type': FETCH_PEERS,
        'fd_type': DATAFEED_PEERS,
        'ticker': ticker,
        'timeframe': '1d',
        'from': iex_utils.last_month().strftime(
            '%Y-%m-%d %H:%M:%S'),
        's3_bucket': s3_bucket_name,
        's3_key': s3_key,
        'redis_key': redis_key,
        's3_enabled': s3_enabled,
        'redis_enabled': redis_enabled
    }

    if label:
        work['label'] = label

    return work
# end of build_iex_fetch_peers_request


def build_iex_fetch_news_request(
        label=None):
    """build_iex_fetch_news_request

    Fetch news data from IEX

    :param label: log label to use
    """
    ticker = TICKER
    base_key = '{}_news_{}'.format(
        ticker,
        datetime.datetime.utcnow().strftime(
            '%Y_%m_%d_%H_%M_%S'))
    s3_bucket_name = NEWS_S3_BUCKET_NAME
    s3_key = base_key
    redis_key = base_key
    s3_enabled = True
    redis_enabled = True

    work = {
        'ft_type': FETCH_NEWS,
        'fd_type': DATAFEED_NEWS,
        'ticker': ticker,
        'timeframe': '1d',
        'from': iex_utils.last_month().strftime(
            '%Y-%m-%d %H:%M:%S'),
        's3_bucket': s3_bucket_name,
        's3_key': s3_key,
        'redis_key': redis_key,
        's3_enabled': s3_enabled,
        'redis_enabled': redis_enabled
    }

    if label:
        work['label'] = label

    return work
# end of build_iex_fetch_news_request


def build_iex_fetch_financials_request(
        label=None):
    """build_iex_fetch_financials_request

    Fetch financial data from IEX

    :param label: log label to use
    """
    ticker = TICKER
    base_key = '{}_financial_{}'.format(
        ticker,
        datetime.datetime.utcnow().strftime(
            '%Y_%m_%d_%H_%M_%S'))
    s3_bucket_name = FINANCIALS_S3_BUCKET_NAME
    s3_key = base_key
    redis_key = base_key
    s3_enabled = True
    redis_enabled = True

    work = {
        'ft_type': FETCH_FINANCIALS,
        'fd_type': DATAFEED_FINANCIALS,
        'ticker': ticker,
        'timeframe': '1d',
        'from': iex_utils.last_month().strftime(
            '%Y-%m-%d %H:%M:%S'),
        's3_bucket': s3_bucket_name,
        's3_key': s3_key,
        'redis_key': redis_key,
        's3_enabled': s3_enabled,
        'redis_enabled': redis_enabled
    }

    if label:
        work['label'] = label

    return work
# end of build_iex_fetch_financials_request


def build_iex_fetch_earnings_request(
        label=None):
    """build_iex_fetch_earnings_request

    Fetch earnings data from IEX

    :param label: log label to use
    """
    ticker = TICKER
    base_key = '{}_earning_{}'.format(
        ticker,
        datetime.datetime.utcnow().strftime(
            '%Y_%m_%d_%H_%M_%S'))
    s3_bucket_name = EARNINGS_S3_BUCKET_NAME
    s3_key = base_key
    redis_key = base_key
    s3_enabled = True
    redis_enabled = True

    work = {
        'ft_type': FETCH_EARNINGS,
        'fd_type': DATAFEED_EARNINGS,
        'ticker': ticker,
        'timeframe': '1d',
        'from': iex_utils.last_month().strftime(
            '%Y-%m-%d %H:%M:%S'),
        's3_bucket': s3_bucket_name,
        's3_key': s3_key,
        'redis_key': redis_key,
        's3_enabled': s3_enabled,
        'redis_enabled': redis_enabled
    }

    if label:
        work['label'] = label

    return work
# end of build_iex_fetch_earnings_request


def build_iex_fetch_dividends_request(
        label=None):
    """build_iex_fetch_dividends_request

    Fetch dividend data from IEX

    :param label: log label to use
    """
    ticker = TICKER
    base_key = '{}_dividend_{}'.format(
        ticker,
        datetime.datetime.utcnow().strftime(
            '%Y_%m_%d_%H_%M_%S'))
    s3_bucket_name = DIVIDENDS_S3_BUCKET_NAME
    s3_key = base_key
    redis_key = base_key
    s3_enabled = True
    redis_enabled = True

    work = {
        'ft_type': FETCH_DIVIDENDS,
        'fd_type': DATAFEED_DIVIDENDS,
        'ticker': ticker,
        'timeframe': '1d',
        'from': iex_utils.last_month().strftime(
            '%Y-%m-%d %H:%M:%S'),
        's3_bucket': s3_bucket_name,
        's3_key': s3_key,
        'redis_key': redis_key,
        's3_enabled': s3_enabled,
        'redis_enabled': redis_enabled
    }

    if label:
        work['label'] = label

    return work
# end of build_iex_fetch_dividends_request


def build_iex_fetch_company_request(
        label=None):
    """build_iex_fetch_company_request

    Fetch company data from IEX

    :param label: log label to use
    """
    ticker = TICKER
    base_key = '{}_company_{}'.format(
        ticker,
        datetime.datetime.utcnow().strftime(
            '%Y_%m_%d_%H_%M_%S'))
    s3_bucket_name = COMPANY_S3_BUCKET_NAME
    s3_key = base_key
    redis_key = base_key
    s3_enabled = True
    redis_enabled = True

    work = {
        'ft_type': FETCH_COMPANY,
        'fd_type': DATAFEED_COMPANY,
        'ticker': ticker,
        'timeframe': '1d',
        'from': iex_utils.last_month().strftime(
            '%Y-%m-%d %H:%M:%S'),
        's3_bucket': s3_bucket_name,
        's3_key': s3_key,
        'redis_key': redis_key,
        's3_enabled': s3_enabled,
        'redis_enabled': redis_enabled
    }

    if label:
        work['label'] = label

    return work
# end of build_iex_fetch_company_request



"""
Parse data from yahoo
"""

import copy
import pinance
import analysis_engine.options_dates
import analysis_engine.get_pricing
import analysis_engine.build_result as build_result
import analysis_engine.work_tasks.publish_pricing_update as \
    publisher
from spylunking.log.setup_logging import build_colorized_logger
from analysis_engine.consts import TICKER
from analysis_engine.consts import SUCCESS
from analysis_engine.consts import NOT_RUN
from analysis_engine.consts import ERR
from analysis_engine.consts import NOT_SET
from analysis_engine.consts import get_status

log = build_colorized_logger(
    name=__name__)


def get_data_from_yahoo(
        work_dict):
    """get_data_from_yahoo

    Get data from yahoo

    :param work_dict: request dictionary
    """
    label = 'get_data_from_yahoo'

    log.info(
        'task - {} - start '
        'work_dict={}'.format(
            label,
            work_dict))

    num_news_rec = 0
    num_options_chains = 0
    cur_high = -1
    cur_low = -1
    cur_open = -1
    cur_close = -1
    cur_volume = -1

    rec = {
        'pricing': None,
        'options': None,
        'news': None,
        'exp_date': None,
        'publish_pricing_update': None,
        'date': None,
        'updated': None
    }
    res = {
        'status': NOT_RUN,
        'err': None,
        'rec': rec
    }

    try:

        ticker = work_dict.get(
            'ticker',
            TICKER)
        exp_date = work_dict.get(
            'exp_date',
            None)
        cur_strike = work_dict.get(
            'strike',
            None)
        contract_type = str(work_dict.get(
            'contract',
            'C')).upper()
        get_pricing = work_dict.get(
            'get_pricing',
            True)
        get_news = work_dict.get(
            'get_news',
            True)
        get_options = work_dict.get(
            'get_options',
            True)
        label = work_dict.get(
            'label',
            label)

        ticker_results = pinance.Pinance(ticker)

        use_date = exp_date
        if not exp_date:
            exp_date = analysis_engine.options_dates.option_expiration(
                 date=exp_date)
            use_date = exp_date.strftime('%Y-%m-%d')

        if get_pricing:
            log.info(
                '{} getting ticker={} pricing'.format(
                    label,
                    ticker))
            ticker_results.get_quotes()
            if ticker_results.quotes_data:
                rec['pricing'] = ticker_results.quotes_data

                cur_high = rec['pricing'].get(
                    'regularMarketDayHigh',
                    None)
                cur_low = rec['pricing'].get(
                    'regularMarketDayLow',
                    None)
                cur_open = rec['pricing'].get(
                    'regularMarketOpen',
                    None)
                cur_close = rec['pricing'].get(
                    'regularMarketPreviousClose',
                    None)
                cur_volume = rec['pricing'].get(
                    'regularMarketVolume',
                    None)
                rec['pricing']['high'] = cur_high
                rec['pricing']['low'] = cur_low
                rec['pricing']['open'] = cur_open
                rec['pricing']['close'] = cur_close
                rec['pricing']['volume'] = cur_volume
            else:
                log.error(
                    '{} ticker={} missing quotes_data'.format(
                        label,
                        ticker))
            # end of if ticker_results.quotes_data

            log.info(
                '{} ticker={} close={} vol={}'.format(
                    label,
                    ticker,
                    cur_close,
                    cur_volume))
        else:
            log.info(
                '{} skip - getting ticker={} pricing'.format(
                    label,
                    ticker,
                    get_pricing))
        # if get_pricing

        if get_news:
            log.info(
                '{} getting ticker={} news'.format(
                    label,
                    ticker))
            ticker_results.get_news()
            if ticker_results.news_data:
                rec['news'] = ticker_results.news_data
            # end of if ticker_results.news_data
        else:
            log.info(
                '{} skip - getting ticker={} news'.format(
                    label,
                    ticker))
        # end if get_news

        if get_options:
            if cur_close:
                cur_strike = int(cur_close)
            if not cur_strike:
                cur_strike = 287

            num_news_rec = 0
            if rec['news']:
                num_news_rec = len(rec['news'])
            log.info(
                '{} ticker={} num_news={} get options close={} '
                'exp_date={} contract={} strike={}'.format(
                    label,
                    ticker,
                    num_news_rec,
                    cur_close,
                    use_date,
                    contract_type,
                    cur_strike))

            rec['options'] = \
                analysis_engine.get_pricing.get_options(
                    ticker=ticker,
                    exp_date_str=use_date,
                    contract_type=contract_type,
                    strike=cur_strike)

            num_options_chains = len(rec['options'])
        else:
            log.info(
                '{} skip - getting ticker={} options'.format(
                    label,
                    ticker))
        # end of if get_options

        log.info(
            '{} yahoo pricing for ticker={} close={} '
            'options={} news={}'.format(
                label,
                ticker,
                cur_close,
                num_options_chains,
                num_news_rec))

        fields_to_upload = [
            'pricing',
            'options',
            'news'
        ]

        for field_name in fields_to_upload:
            upload_and_cache_req = copy.deepcopy(work_dict)
            upload_and_cache_req['celery_disabled'] = True
            upload_and_cache_req['data'] = rec[field_name]
            if not upload_and_cache_req['data']:
                upload_and_cache_req['data'] = '{}'

            if 'redis_key' in work_dict:
                upload_and_cache_req['redis_key'] = '{}_{}'.format(
                    work_dict.get(
                        'redis_key',
                        '{}_{}'.format(
                            ticker,
                            field_name)),
                    field_name)
            if 's3_key' in work_dict:
                upload_and_cache_req['s3_key'] = '{}_{}'.format(
                    work_dict.get(
                        's3_key',
                        '{}_{}'.format(
                            ticker,
                            field_name)),
                    field_name)
            try:
                update_res = publisher.run_publish_pricing_update(
                    work_dict=upload_and_cache_req)
                update_status = update_res.get(
                    'status',
                    NOT_SET)
                log.info(
                    '{} publish update status={} data={}'.format(
                        label,
                        get_status(status=update_status),
                        update_res))
            except Exception as f:
                err = (
                    '{} - failed to upload YAHOO data={} to '
                    'to s3_key={} and redis_key={}'.format(
                        label,
                        upload_and_cache_req,
                        upload_and_cache_req['s3_key'],
                        upload_and_cache_req['redis_key']))
                log.error(err)
            # end of try/ex to upload and cache
            if not rec[field_name]:
                log.debug(
                    '{} - ticker={} no data from YAHOO for '
                    'field_name={}'.format(
                        label,
                        ticker,
                        field_name))
        # end of for all fields

        res = build_result.build_result(
            status=SUCCESS,
            err=None,
            rec=rec)

    except Exception as e:
        res = build_result.build_result(
            status=ERR,
            err=(
                'failed - get_data_from_yahoo '
                'dict={} with ex={}').format(
                    work_dict,
                    e),
            rec=rec)
        log.error(
            '{} - {}'.format(
                label,
                res['err']))
    # end of try/ex

    log.info(
        'task - get_data_from_yahoo done - '
        '{} - status={}'.format(
            label,
            get_status(res['status'])))

    return res
# end of get_data_from_yahoo



"""
Mock Pinance Object for unittests
"""

import os


def mock_get_options(
        ticker=None,
        contract_type=None,
        exp_date_str=None,
        strike=None):
    """mock_get_options

    :param ticker: ticker to lookup
    :param exp_date_str: ``YYYY-MM-DD`` expiration date format
    :param strike: optional strike price, ``None`` returns
                    all option chains
    :param contract_type: ``C`` calls or ``P`` for puts, if
                            ``strike=None`` then the ``contract_type``
                            is ignored
    """
    options_data = os.getenv(
        'TEST_OPTIONS',
        {
            'openInterest': 0,
            'contractSize': 'REGULAR',
            'change': 0.0,
            'lastTradeDate': 1493323145,
            'impliedVolatility': 0.2500075,
            'strike': 286.0,
            'bid': 0.0,
            'inTheMoney': False,
            'lastPrice': 0.28,
            'percentChange': 0.0,
            'contractSymbol': 'SPY170505C00015000',
            'expiration': 1493942400,
            'volume': 1106,
            'ask': 0.0,
            'currency': 'USD'
        })
    return [
        options_data
    ]
# end of mock_get_options


class MockPinance:
    """MockPinance"""

    def __init__(
            self,
            symbol='SPY'):
        """__init__

        :param symbol: ticker
        """

        self.symbol = symbol
        self.quotes_data = {
            'language': 'en-US',
            'region': 'US',
            'quoteType': 'ETF',
            'quoteSourceName': 'Delayed Quote',
            'currency': 'USD',
            'postMarketChangePercent': 0.06941398,
            'postMarketTime': 1536623987,
            'fiftyDayAverageChangePercent': 0.010071794,
            'sourceInterval': 15,
            'exchangeTimezoneName': 'America/New_York',
            'exchangeTimezoneShortName': 'EDT',
            'gmtOffSetMilliseconds': -14400000,
            'shortName': 'SPDR S&P 500',
            'exchange': 'PCX',
            'marketState': 'POSTPOST',
            'priceHint': 2,
            'regularMarketPreviousClose': 287.6,
            'ask': 0.0,
            'bidSize': 10,
            'askSize': 8,
            'messageBoardId': 'finmb_6160262',
            'fullExchangeName': 'NYSEArca',
            'longName': 'SPDR S&amp;P 500 ETF',
            'financialCurrency': 'USD',
            'averageDailyVolume3Month': 64572187,
            'postMarketPrice': 288.3,
            'postMarketChange': 0.19998169,
            'regularMarketChangePercent': 0.17037213,
            'regularMarketDayRange': '287.88 - 289.03',
            'bid': 0.0,
            'market': 'us_market',
            'regularMarketPrice': 288.09,
            'regularMarketTime': 1536609602,
            'regularMarketChange': 0.48999023,
            'regularMarketOpen': 288.74,
            'regularMarketDayHigh': 289.03,
            'regularMarketDayLow': 287.88,
            'regularMarketVolume': 50210903,
            'esgPopulated': False,
            'tradeable': True,
            'fiftyTwoWeekHighChangePercent': -0.012511119,
            'fiftyTwoWeekLow': 248.02,
            'fiftyTwoWeekHigh': 291.74,
            'twoHundredDayAverage': 274.66153,
            'twoHundredDayAverageChange': 13.428467,
            'twoHundredDayAverageChangePercent': 0.048890963,
            'sharesOutstanding': 944232000,
            'fiftyDayAverage': 285.21735,
            'fiftyDayAverageChange': 2.8726501,
            'marketCap': 272023797760,
            'averageDailyVolume10Day': 67116380,
            'fiftyTwoWeekLowChange': 40.069992,
            'fiftyTwoWeekLowChangePercent': 0.16155952,
            'fiftyTwoWeekRange': '248.02 - 291.74',
            'fiftyTwoWeekHighChange': -3.649994,
            'exchangeDataDelayedBy': 0,
            'ytdReturn': 9.84,
            'trailingThreeMonthReturns': 7.63,
            'trailingThreeMonthNavReturns': 7.71,
            'symbol': 'SPY'
        }
        self.news_env_values = os.getenv(
            'TEST_NEWS',
            None)
        self.news_data = [
            {
                'u': 'http://finance.yahoo.com/news/url1',
                'usg': 'ke1',
                'sru': 'http://news.google.com/news/url?values',
                's': 'Yahoo Finance',
                'tt': '1493311950',
                'sp': 'Some Title 1',
                'd': '16 hours ago',
                't': 'Some Title 1'
            },
            {
                'u': 'http://finance.yahoo.com/news/url2',
                'usg': 'key2',
                'sru': 'http://news.google.com/news/url?values',
                's': 'Yahoo Finance',
                'tt': '1493311950',
                'sp': 'Some Title 2',
                'd': '18 hours ago',
                't': 'Some Title 2'
            }
        ]

        self.options_data = os.getenv(
            'TEST_OPTIONS',
            {
                'openInterest': 0,
                'contractSize': 'REGULAR',
                'change': 0.0,
                'lastTradeDate': 1493323145,
                'impliedVolatility': 0.2500075,
                'strike': 286.0,
                'bid': 0.0,
                'inTheMoney': False,
                'lastPrice': 0.28,
                'percentChange': 0.0,
                'contractSymbol': 'SPY170505C00015000',
                'expiration': 1493942400,
                'volume': 1106,
                'ask': 0.0,
                'currency': 'USD'
            })
    # end of __init__

    def get_quotes(
            self):
        """get_quotes"""
        return self.quotes_data
    # end of get_quotes

    def get_news(
            self):
        """get_news"""
        return self.news_data
    # end of get_news

    def get_options(
            self,
            ticker=None,
            contract_type=None,
            exp_date_str=None,
            strike=None):
        """get_options

        :param ticker: ticker to lookup
        :param exp_date_str: ``YYYY-MM-DD`` expiration date format
        :param strike: optional strike price, ``None`` returns
                       all option chains
        :param contract_type: ``C`` calls or ``P`` for puts, if
                              ``strike=None`` then the ``contract_type``
                              is ignored
        """
        return [
            self.options_data
        ]
    # end of get_options

# end of MockPinance
